-- name:\\#f7a5b5\\[CS] Dudaw Kirby
-- description: The Original Puffball!\n\nThis character mod brings Dudaw's Kirby model from SM64: Kirby Edition into Coop Deluxe, with recolorability, and various features from the original hack restored.\n\nSpecial Thanks to I'mYourCat for assisting with coding.

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
]]

-- Replace Mod Name with your Character/Pack name.
local TEXT_MOD_NAME = "Dudaw Kirby"

-- Stops mod from loading if Character Select isn't on, Does not need to be touched
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end


local E_MODEL_DKIRBY =      smlua_model_util_get_id("dudawkirby_geo")      -- Located in "actors"
local E_MODEL_WARPSTAR =      smlua_model_util_get_id("warpstar_geo") 
local E_MODEL_D1UP = smlua_model_util_get_id("mushroom_dkirby1up_geo") 

local TEX_DKIRBY_LIFE_ICON = get_texture_info("dudawkirby-icon") -- Located in "textures"
-- local TEX_CHAR_STAR_ICON = get_texture_info("exclamation-icon") -- Located in "textures"

-- All sound files are located in "sound" folder
-- Remember to include the file extention in the name
local VOICETABLE_DKIRBY = {
    [CHAR_SOUND_OKEY_DOKEY] =        'kirbyokeydokey.ogg', -- Starting game
	[CHAR_SOUND_LETS_A_GO] =         nil, -- Starting level
	[CHAR_SOUND_PUNCH_YAH] =         'kirbyjump-hah.ogg', -- Punch 1
	[CHAR_SOUND_PUNCH_WAH] =         'kirby-wahpunch.ogg', -- Punch 2
	[CHAR_SOUND_PUNCH_HOO] =         'kirby-hoo.ogg', -- Punch 3
	[CHAR_SOUND_YAH_WAH_HOO] =       {'kirbyjump-wah.ogg', 'kirbyjump-wuh.ogg', 'kirbyjump-hah.ogg'}, -- First Jump Sounds
	[CHAR_SOUND_HOOHOO] =            'kirby-hoo.ogg', -- Second jump sound
	[CHAR_SOUND_YAHOO_WAHA_YIPPEE] = 'kirby-triplelongjump.ogg', -- Triple jump sounds
	[CHAR_SOUND_UH] =                nil, -- Wall bonk
	[CHAR_SOUND_UH2] =               'kirby-hoo.ogg', -- Landing after long jump
	[CHAR_SOUND_UH2_2] =             nil,
	[CHAR_SOUND_HAHA] =              nil, -- Landing triple jump
	[CHAR_SOUND_YAHOO] =             'kirby-triplelongjump.ogg', -- Long jump
	[CHAR_SOUND_OOOF] = "kirby-ouch.ogg",
    [CHAR_SOUND_OOOF2] = "kirby-ouch.ogg",
	[CHAR_SOUND_DOH] =               'kirby-ouch.ogg', -- Long jump wall bonk
	[CHAR_SOUND_WHOA] =              'kirby-hoo.ogg', -- Grabbing ledge
	[CHAR_SOUND_EEUH] =              'kirby-hoo.ogg', -- Climbing over ledge
	[CHAR_SOUND_WAAAOOOW] =          'kirby-fall.ogg', -- Falling a long distance
	[CHAR_SOUND_TWIRL_BOUNCE] =      nil, -- Bouncing off of a flower spring
	[CHAR_SOUND_GROUND_POUND_WAH] =  'kirbyjump-hah.ogg', 
	[CHAR_SOUND_HRMM] =              'kirby-pickup.ogg', -- Lifting something
	[CHAR_SOUND_HERE_WE_GO] =        nil, -- Star get
	[CHAR_SOUND_SO_LONGA_BOWSER] =   nil, -- Throwing Bowser
--DAMAGE
	[CHAR_SOUND_ATTACKED] = 'kirby-ouch.ogg', -- Damaged
	[CHAR_SOUND_PANTING] = 'kirby-panting.ogg', -- Low health
	[CHAR_SOUND_ON_FIRE] = 'kirby-fall.ogg', -- Burned
--SLEEP SOUNDS
	[CHAR_SOUND_IMA_TIRED] = nil, -- Mario feeling tired
	[CHAR_SOUND_YAWNING] = nil, -- Mario yawning before he sits down to sleep
	[CHAR_SOUND_SNORING1] = nil, -- Snore Inhale
	[CHAR_SOUND_SNORING2] = nil, -- Exhale
	[CHAR_SOUND_SNORING3] = nil, -- Sleep talking / mumbling
--COUGHING (USED IN THE GAS MAZE)
	[CHAR_SOUND_COUGHING1] = 'kirby-cough.ogg', -- Cough take 1
	[CHAR_SOUND_COUGHING2] = 'kirby-cough.ogg', -- Cough take 2
	[CHAR_SOUND_COUGHING3] = 'kirby-cough.ogg', -- Cough take 3
--DEATH
	[CHAR_SOUND_DYING] = 'kirby-ouch.ogg', -- Dying from damage
	[CHAR_SOUND_DROWNING] = 'kirby-ouch.ogg', -- Running out of air underwater
	[CHAR_SOUND_MAMA_MIA] = 'kirby-mamamia.ogg' -- Booted out of level
}

-- All Located in "actors" folder
-- (Models do not exist in template)
--[[
local CAPTABLE_CHAR = {
    normal = smlua_model_util_get_id("FILENAME_geo"),
    wing = smlua_model_util_get_id("FILENAME_geo"),
    metal = smlua_model_util_get_id("FILENAME_geo"),
}
]]

local PALETTE_DKIRBY = {
    [PANTS]  = "0000ff",
    [SHIRT]  = "ff0000",
    [GLOVES] = "ffffff",
    [SHOES]  = "ff0000",
    [HAIR]   = "730600",
    [SKIN]   = "f7a5b5",
    [CAP]    = "ff0000",
	[EMBLEM] = "ff0000"
}

-- All Located in "textures" folder
-- (Textures do not exist in template)
local HEALTH_METER_DKIRBY = {
    label = {
        left = get_texture_info("kirbyhp-half1"),
        right = get_texture_info("kirbyhp-half2"),
    },
    pie = {
        [1] = get_texture_info("kirbyhp-1"),
        [2] = get_texture_info("kirbyhp-2"),
        [3] = get_texture_info("kirbyhp-3"),
        [4] = get_texture_info("kirbyhp-4"),
        [5] = get_texture_info("kirbyhp-5"),
        [6] = get_texture_info("kirbyhp-6"),
        [7] = get_texture_info("kirbyhp-7"),
        [8] = get_texture_info("kirbyhp-8"),
    }
}


local TEX_DKIRBY_COURSE_TOP = get_texture_info("dudawkirby-course1")
local TEX_DKIRBY_COURSE_BOTTOM = get_texture_info("dudawkirby-course2")

local DKIRBYCourseInfo = {
    top = TEX_DKIRBY_COURSE_TOP,
    bottom = TEX_DKIRBY_COURSE_BOTTOM,
}

local CSloaded = false
local function on_character_select_load()
    -- Adds the custom character to the Character Select Menu
    CT_DKIRBY = _G.charSelect.character_add(
        "Dudaw Kirby", -- Character Name
        "This the Kirby model from SM64: Kirby Edition, one the first custom character romhacks to be released. When playing as this Kirby, various features from said hack come with him. He's got a Warp Star replacing the Koopa Shells, his own Power-up music. Many other texture replacements. And recolorability!", -- Description
        "Dudaw12", -- Credits
        "f7a5b5",           -- Menu Color
        E_MODEL_DKIRBY,       -- Character Model
        CT_MARIO,           -- Override Character
        TEX_DKIRBY_LIFE_ICON, -- Life Icon
        1,                  -- Camera Scale
        0                   -- Vertical Offset
    )

    -- Adds a voice to your character
    -- (Sounds do not exist in template)
    _G.charSelect.character_add_voice(E_MODEL_DKIRBY, VOICETABLE_DKIRBY)

    -- Adds a palette to your character
    _G.charSelect.character_add_palette_preset(E_MODEL_DKIRBY, PALETTE_DKIRBY)

    -- Adds a health meter to your character
    -- (Textures do not exist in template)
    _G.charSelect.character_add_health_meter(CT_DKIRBY, HEALTH_METER_DKIRBY)
	_G.charSelect.character_add_course_texture(CT_DKIRBY, DKIRBYCourseInfo)
    -- Adds credits to the credits menu
    _G.charSelect.credit_add(TEXT_MOD_NAME, "B. Bones Johnson", "Dudaw Kirby Port")
	_G.charSelect.credit_add(TEXT_MOD_NAME, "I'mYourCat", "Coding Assistance")
	
    CSloaded = true
end


local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_DKIRBY then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_DKIRBY then return _G.charSelect.voice.snore(m) end
end


local function for_each_object_with_behavior(behavior, func) -- by isaac
    local o = obj_get_first_with_behavior_id(behavior)
    while o do
        func(o)
        o = obj_get_next_with_same_behavior_id(o)
    end
end

local function kirbystuff(m)

    local currentCharacter = _G.charSelect.character_get_current_number(0)
    local mRiddenObj = m.riddenObj

    if m.playerIndex ~= 0 then return end

--Kirby Stuff Handler
	
	--Dudaw Kirby Idle
	
	if m.action == ACT_IDLE and currentCharacter == CT_DKIRBY then
	m.marioObj.header.gfx.animInfo.curAnim = get_mario_vanilla_animation(MARIO_ANIM_FIRST_PERSON)
	end
	
	--Warp Star Code
	

	
	    for_each_object_with_behavior(id_bhvKoopaShell, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_WARPSTAR)
    else
        obj_set_model_extended(b, E_MODEL_KOOPA_SHELL)
    end
end)
	
-- 1-Up Mushroom Code

    for_each_object_with_behavior(id_bhv1Up, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhv1upWalking, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhv1upRunningAway, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhv1upSliding, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhv1upJumpOnApproach, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhvHidden1up, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

    for_each_object_with_behavior(id_bhvHidden1upInPole, function(b)
    if currentCharacter == CT_DKIRBY then
        obj_set_model_extended(b, E_MODEL_D1UP)
    else
        obj_set_model_extended(b, E_MODEL_1UP)
    end
end)

end
	


local SEQ_KIRBY_POWERUP = SEQ_COUNT + 4
local SEQ_KIRBY_METAL = SEQ_COUNT + 8

smlua_audio_utils_replace_sequence(SEQ_KIRBY_POWERUP, 0x0B, 75, "kirbypower")
smlua_audio_utils_replace_sequence(SEQ_KIRBY_METAL, 0x0B, 75, "kirbypower")

local function replace_seq(p, seq, l)
    if _G.charSelect.character_get_current_number() == CT_DKIRBY then
        if seq == SEQ_EVENT_POWERUP then
            return SEQ_KIRBY_POWERUP
        elseif seq == SEQ_EVENT_METAL_CAP then
            return SEQ_KIRBY_METAL
        end
		else return seq
    end
end

hook_event(HOOK_ON_SEQ_LOAD, replace_seq)


hook_event(HOOK_MARIO_UPDATE, kirbystuff)
hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)